import { IImperativeError, Logger, RestClient, Session } from "@brightside/imperative";
import { IBufferWithEtag } from "../../zosfiles/src/api/doc/IBufferWithEtagResponse";
import { IStringWithEtag } from "../../zosfiles/src/api/doc/IStringWithEtagResponse";
/**
 * Wrapper for invoke z/OSMF API through the RestClient to perform common error
 * handling and checking and resolve promises according to generic types
 * @export
 * @class ZosmfRestClient
 * @extends {RestClient}
 */
export declare class ZosmfRestClient extends RestClient {
    /**
     * Use the Brightside logger instead of the imperative logger
     * @type {Logger}
     */
    readonly log: Logger;
    /**
     * REST HTTP GET operation
     * @static
     * @param {Session} session - representing connection to this api
     * @param {string} resource - URI for which this request should go against
     * @param {any} reqHeaders - headers to include in the REST request
     * @returns {Promise<IBufferWithEtag>} - response body content from http(s) call and ETag value
     * @throws  if the request gets a status code outside of the 200 range
     *          or other connection problems occur (e.g. connection refused)
     * @memberof ZosmfRestClient
     */
    static getExpectBufferAndEtag(session: Session, resource: string, reqHeaders?: any[]): Promise<IBufferWithEtag>;
    /**
     * REST HTTP PUT operation
     * @static
     * @param {AbstractSession} session - representing connection to this api
     * @param {string} resource - URI for which this request should go against
     * @param {object[]} reqHeaders - headers to include in the REST request
     * @param {any} data - payload data
     * @returns {Promise<IStringWithEtag>} - response body content from http(s) call
     * @throws  if the request gets a status code outside of the 200 range
     *          or other connection problems occur (e.g. connection refused)
     * @memberof RestClient
     */
    static putExpectStringAndEtag(session: Session, resource: string, reqHeaders: any[], data: any): Promise<IStringWithEtag>;
    /**
     * Append z/OSMF specific headers to the callers headers for cases
     * where a header is common to every request.
     * @param {any[] | undefined} headers - current header array
     * @memberof ZosmfRestClient
     */
    protected appendHeaders(headers: any[] | undefined): any[];
    /**
     * Process an error encountered in the rest client
     * @param {IImperativeError} original - the original error automatically built by the abstract rest client
     * @returns {IImperativeError} - the processed error with details added
     * @memberof ZosmfRestClient
     */
    protected processError(original: IImperativeError): IImperativeError;
}
