"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
Object.defineProperty(exports, "__esModule", { value: true });
const jsonfile_1 = require("jsonfile");
const fs_1 = require("fs");
const logger_1 = require("../../logger");
/**
 * This class represents settings for an Imperative CLI application that can be configured
 * by an end user by modifying a settings file. Settings are stored in {@link AppSettings#settings}
 * in a format specified by {@link ISettingsFile}.
 */
class AppSettings {
    /**
     * Constructs a new settings object
     *
     * @param settingsFile  The full path to a settings file to load.
     * @param missingFileRecovery A recovery function for when the settings file didn't exist
     */
    constructor(settingsFile, missingFileRecovery) {
        this.settingsFile = settingsFile;
        let settings;
        // NOTE: All overrides key values should be false in the default object.
        const defaultSettings = {
            overrides: {
                CredentialManager: false
            }
        };
        try {
            // Try to load the file immediately, if it fails we will then
            // try to recover
            logger_1.Logger.getImperativeLogger().trace(`Attempting to load settings file: ${settingsFile}`);
            settings = jsonfile_1.readFileSync(settingsFile);
            logger_1.Logger.getImperativeLogger().trace("Settings were loaded");
        }
        catch (up) {
            if (missingFileRecovery && !fs_1.existsSync(settingsFile)) {
                logger_1.Logger.getImperativeLogger().trace("Executing missing file recovery.");
                settings = missingFileRecovery(settingsFile, defaultSettings);
            }
            else {
                logger_1.Logger.getImperativeLogger().error("Unable to recover from load failure");
                logger_1.Logger.getImperativeLogger().error(up.toString());
                // Throw up if there is no recovery function or there was a recovery
                // function but the file already existed. (Indicates there was a bigger
                // issue at play)
                throw up;
            }
        }
        // Merge objects loaded recursively
        const DeepMerge = require("deepmerge");
        this.settings = DeepMerge(defaultSettings, settings);
        logger_1.Logger.getImperativeLogger().trace("Loaded Settings:");
        logger_1.Logger.getImperativeLogger().trace(this.settings); // This works because someone does the object translation
    }
    /**
     *
     * @param settingsFile The settings file to load from.
     * @param missingFileRecovery A recovery function when the settings file isn't found
     *
     * @returns A reference to the created instance. This is the same instance returned by {@link AppSettings.instance}
     *
     * @throws {@link SettingsAlreadyInitialized} When the settings singleton has previously been initialized.
     */
    static initialize(settingsFile, missingFileRecovery) {
        if (AppSettings.mInstance) {
            // Throw an error imported at runtime so that we minimize file that get included
            // on startup.
            const { SettingsAlreadyInitialized } = require("./errors/index");
            throw new SettingsAlreadyInitialized();
        }
        AppSettings.mInstance = new AppSettings(settingsFile, missingFileRecovery);
        return AppSettings.mInstance;
    }
    /**
     * Get the singleton instance of the app settings object that was initialized
     * within the {@link AppSettings.initialize} function.
     *
     * @returns A singleton AppSettings object
     *
     * @throws {@link SettingsNotInitialized} When the settings singleton has not been initialized.
     */
    static get instance() {
        if (AppSettings.mInstance == null) {
            // Throw an error imported at runtime so that we minimize file that get included
            // on startup.
            const { SettingsNotInitialized } = require("./errors/index");
            throw new SettingsNotInitialized();
        }
        return AppSettings.mInstance;
    }
    /**
     * Set a new override and save it to the settings file.
     *
     * @param override The override being set
     * @param value The new value
     *
     * @returns A promise of completion
     */
    setNewOverride(override, value) {
        this.settings.overrides[override] = value;
        return this.writeSettingsFile();
    }
    /**
     * @returns true if the app settings have been initialized
     */
    static get initialized() {
        return !(this.mInstance == null);
    }
    /**
     * Writes settings to the file.
     *
     * @returns A promise of completion
     */
    writeSettingsFile() {
        return new Promise((resolve, reject) => {
            jsonfile_1.writeFile(this.settingsFile, this.settings, {
                spaces: 2
            }, (err) => {
                if (err) {
                    logger_1.Logger.getImperativeLogger().error("Unable to save settings");
                    logger_1.Logger.getImperativeLogger().error(err.toString());
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        });
    }
}
exports.AppSettings = AppSettings;
//# sourceMappingURL=AppSettings.js.map