"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable: max-classes-per-file
const imperative_1 = require("@brightside/imperative");
const nls = require("vscode-nls");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const zowe = require("@brightside/core");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const extension_1 = require("./extension"); // TODO: resolve cyclic dependency
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
let IConnection;
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.profilesByType = new Map();
        this.defaultProfileByType = new Map();
        this.profileManagerByType = new Map();
    }
    // Processing stops if there are no profiles detected
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    static credentialHashSetValue(profile, property, value) {
        let properties = this.credentialsHash.get(profile);
        if (!properties) {
            properties = new Map();
            this.credentialsHash.set(profile, properties);
        }
        properties.set(property, value);
    }
    loadNamedProfile(name, type) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && (type ? profile.type === type : true)) {
                return profile;
            }
        }
        throw new Error(localize(0, null)
            + name + localize(1, null));
    }
    getDefaultProfile(type = "zosmf") {
        return this.defaultProfileByType.get(type);
    }
    getProfiles(type = "zosmf") {
        return this.profilesByType.get(type);
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allProfiles = [];
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                const profilesForType = (yield profileManager.loadAll()).filter((profile) => {
                    return profile.type === type;
                });
                if (profilesForType && profilesForType.length > 0) {
                    this.allProfiles.push(...profilesForType);
                    this.profilesByType.set(type, profilesForType);
                    this.defaultProfileByType.set(type, (yield profileManager.load({ loadDefault: true })));
                }
            }
            // TODO: Temporary hack to be consistent with sessions storing prompted passwords.
            // Should be rewritten and a clear method needs to be added.
            for (const profile of this.allProfiles) {
                const credentialProps = Profiles.credentialsHash.get(profile.name);
                if (credentialProps) {
                    profile.profile.user = credentialProps.get(Profiles.credentialHashPropUsername);
                    profile.profile.password = credentialProps.get(Profiles.credentialHashPropPassword);
                }
            }
        });
    }
    validateAndParseUrl(newUrl) {
        let url;
        const validProtocols = ["https"];
        const DEFAULT_HTTPS_PORT = 443;
        const validationResult = {
            valid: false,
            host: null,
            port: null
        };
        try {
            url = new url_1.URL(newUrl);
        }
        catch (error) {
            return validationResult;
        }
        // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
        if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
            return validationResult;
        }
        // if port is empty, set https defaults
        if (!url.port.trim()) {
            validationResult.port = DEFAULT_HTTPS_PORT;
        }
        else {
            validationResult.port = Number(url.port);
        }
        validationResult.host = url.hostname;
        validationResult.valid = true;
        return validationResult;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => { resolve(urlInputBox.value); });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage = localize(2, null);
                    }
                });
            });
        });
    }
    createNewConnection(profileName, profileType = "zosmf") {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let zosmfURL;
            let rejectUnauthorize;
            let options;
            const urlInputBox = vscode.window.createInputBox();
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = localize(3, null);
            urlInputBox.prompt = localize(4, null);
            urlInputBox.show();
            zosmfURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!zosmfURL) {
                vscode.window.showInformationMessage(localize(5, null));
                return undefined;
            }
            const zosmfUrlParsed = this.validateAndParseUrl(zosmfURL);
            options = {
                placeHolder: localize(6, null),
                prompt: localize(7, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(options);
            if (userName === undefined) {
                vscode.window.showInformationMessage(localize(8, null));
                return;
            }
            options = {
                placeHolder: localize(9, null),
                prompt: localize(10, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(options);
            if (passWord === undefined) {
                vscode.window.showInformationMessage(localize(11, null));
                return;
            }
            const quickPickOptions = {
                placeHolder: localize(12, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const selectRU = ["True - Reject connections with self-signed certificates",
                "False - Accept connections with self-signed certificates"];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                vscode.window.showInformationMessage(localize(13, null));
                return undefined;
            }
            for (const profile of this.allProfiles) {
                if (profile.name === profileName) {
                    vscode.window.showErrorMessage(localize(14, null));
                    return undefined;
                }
            }
            IConnection = {
                name: profileName,
                host: zosmfUrlParsed.host,
                port: zosmfUrlParsed.port,
                user: userName,
                password: passWord,
                rejectUnauthorized: rejectUnauthorize
            };
            let newProfile;
            try {
                newProfile = yield this.saveProfile(IConnection, IConnection.name, profileType);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            yield zowe.ZosmfSession.createBasicZosmfSession(newProfile);
            vscode.window.showInformationMessage("Profile " + profileName + " was created.");
            return profileName;
        });
    }
    promptCredentials(sessName) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let options;
            const loadProfile = this.loadNamedProfile(sessName);
            const loadSession = loadProfile.profile;
            if (!loadSession.user) {
                options = {
                    placeHolder: localize(15, null),
                    prompt: localize(16, null),
                    value: userName
                };
                userName = yield vscode.window.showInputBox(options);
                if (!userName) {
                    vscode.window.showErrorMessage(localize(17, null));
                    return;
                }
                else {
                    loadSession.user = loadProfile.profile.user = userName;
                    Profiles.credentialHashSetValue(sessName, Profiles.credentialHashPropUsername, userName);
                }
            }
            if (!loadSession.password) {
                passWord = loadSession.password;
                options = {
                    placeHolder: localize(18, null),
                    prompt: localize(19, null),
                    password: true,
                    value: passWord
                };
                passWord = yield vscode.window.showInputBox(options);
                if (!passWord) {
                    vscode.window.showErrorMessage(localize(20, null));
                    return;
                }
                else {
                    loadSession.password = loadProfile.profile.password = passWord.trim();
                    Profiles.credentialHashSetValue(sessName, Profiles.credentialHashPropPassword, loadSession.password);
                }
            }
            const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(loadSession);
            return [updSession.ISession.user, updSession.ISession.password, updSession.ISession.base64EncodedAuth];
        });
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfProfile;
            try {
                zosmfProfile = yield (yield this.getCliProfileManager(ProfileType)).save({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    getCliProfileManager(type) {
        return __awaiter(this, void 0, void 0, function* () {
            let profileManager = this.profileManagerByType.get(type);
            if (!profileManager) {
                profileManager = yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(extension_1.getZoweDir(), "profiles"),
                    type
                });
                if (profileManager) {
                    this.profileManagerByType.set(type, profileManager);
                }
                else {
                    return undefined;
                }
            }
            return profileManager;
        });
    }
}
exports.Profiles = Profiles;
// TODO: Temporary hack for creditials entered via user prompts to survive a refresh.
// The way credentials are prompted and queried needs to be rewritten and a clear method added.
Profiles.credentialsHash = new Map();
Profiles.credentialHashPropUsername = "user";
Profiles.credentialHashPropPassword = "password";

//# sourceMappingURL=Profiles.js.map
