"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const core_1 = require("@brightside/core");
// tslint:disable-next-line: no-duplicate-imports
const Profiles_1 = require("./Profiles");
const PersistentFilters_1 = require("./PersistentFilters");
const ZoweJobNode_1 = require("./ZoweJobNode");
const utils_1 = require("./utils");
const extension = require("../src/extension");
const nls = require("vscode-nls");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Creates the Job tree that contains nodes of sessions, jobs and spool items
 *
 * @export
 * @class ZosJobsProvider
 * @implements {vscode.TreeDataProvider}
 */
function createJobsTree(log) {
    return __awaiter(this, void 0, void 0, function* () {
        const tree = new ZosJobsProvider();
        yield tree.initializeJobsTree(log);
        yield tree.addSession();
        return tree;
    });
}
exports.createJobsTree = createJobsTree;
// tslint:disable-next-line: max-classes-per-file
class ZosJobsProvider {
    constructor() {
        this.mSessionNodes = [];
        this.mFavorites = [];
        // Event Emitters used to notify subscribers that the refresh event has fired
        this.mOnDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this.mOnDidChangeTreeData.event;
        this.createOwner = new utils_1.OwnerFilterDescriptor();
        this.createId = new utils_1.JobIdFilterDescriptor();
        this.validProfile = -1;
        this.mFavoriteSession = new ZoweJobNode_1.Job(localize(0, null), vscode.TreeItemCollapsibleState.Collapsed, null, null, null, null);
        this.mFavoriteSession.contextValue = extension.FAVORITE_CONTEXT;
        this.mFavoriteSession.iconPath = utils_1.applyIcons(this.mFavoriteSession);
        this.mSessionNodes = [this.mFavoriteSession];
        this.mHistory = new PersistentFilters_1.PersistentFilters(ZosJobsProvider.persistenceSchema);
    }
    getChildren(element) {
        if (element) {
            // solution for optional credentials. Owner is having error on initialization.
            if (element.owner === "") {
                return;
            }
            if (element.contextValue === extension.FAVORITE_CONTEXT) {
                return this.mFavorites;
            }
            return element.getChildren();
        }
        return this.mSessionNodes;
    }
    getTreeItem(element) {
        return element;
    }
    getParent(element) {
        return element.mParent;
    }
    /**
     * Adds a session to the data set tree
     *
     * @param {string} [sessionName] - optional; loads default profile if not passed
     */
    addSession(sessionName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads profile associated with passed sessionName, default if none passed
            if (sessionName) {
                const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sessionName);
                if (zosmfProfile) {
                    this.addSingleSession(zosmfProfile);
                }
            }
            else {
                const zosmfProfiles = Profiles_1.Profiles.getInstance().allProfiles;
                for (const zosmfProfile of zosmfProfiles) {
                    // If session is already added, do nothing
                    if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                        continue;
                    }
                    for (const session of this.mHistory.getSessions()) {
                        if (session === zosmfProfile.name) {
                            this.addSingleSession(zosmfProfile);
                        }
                    }
                }
                if (this.mSessionNodes.length === 1) {
                    this.addSingleSession(Profiles_1.Profiles.getInstance().getDefaultProfile());
                }
            }
            this.refresh();
        });
    }
    deleteSession(node) {
        this.mSessionNodes = this.mSessionNodes.filter((tempNode) => tempNode.label.trim() !== node.label.trim());
        let revisedLabel = node.label;
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
    deleteJob(node) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(node.profile).deleteJob(node.job.jobname, node.job.jobid);
                vscode.window.showInformationMessage(localize(1, null) + node.job.jobname + "(" + node.job.jobid + ")" +
                    localize(2, null));
                this.removeJobsFavorite(this.createJobsFavorite(node));
            }
            catch (error) {
                yield utils_1.errorHandling(error, node.getProfileName(), error.message);
            }
        });
    }
    /**
     * Selects a specific job in the Jobs view
     *
     * @param {Job}
     */
    setJob(treeView, job) {
        treeView.reveal(job, { select: true, focus: true });
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refreshElement(element) {
        element.dirty = true;
        this.mOnDidChangeTreeData.fire(element);
    }
    /**
     * Called whenever the tree needs to be refreshed, and fires the data change event
     *
     */
    refresh() {
        this.mOnDidChangeTreeData.fire();
    }
    /**
     * Change the state of an expandable node
     * @param provider the tree view provider
     * @param element the node being flipped
     * @param isOpen the intended state of the the tree view provider, true or false
     */
    flipState(element, isOpen = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element.label !== "Favorites") {
                let usrNme;
                let passWrd;
                let baseEncd;
                let sesNamePrompt;
                if (element.contextValue.endsWith(extension.FAV_SUFFIX)) {
                    sesNamePrompt = element.label.substring(1, element.label.indexOf("]"));
                }
                else {
                    sesNamePrompt = element.label;
                }
                if ((!element.session.ISession.user) || (!element.session.ISession.password)) {
                    try {
                        const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                        if (values !== undefined) {
                            usrNme = values[0];
                            passWrd = values[1];
                            baseEncd = values[2];
                        }
                    }
                    catch (error) {
                        yield utils_1.errorHandling(error, element.getProfileName(), error.message);
                    }
                    if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                        element.session.ISession.user = usrNme;
                        element.session.ISession.password = passWrd;
                        element.session.ISession.base64EncodedAuth = baseEncd;
                        element.owner = usrNme;
                        this.validProfile = 1;
                    }
                    else {
                        return;
                    }
                    yield this.refreshElement(element);
                    yield this.refresh();
                }
                else {
                    this.validProfile = 1;
                }
            }
            else {
                this.validProfile = 1;
            }
            if (this.validProfile === 1) {
                element.iconPath = utils_1.applyIcons(element, isOpen ? extension.ICON_STATE_OPEN : extension.ICON_STATE_CLOSED);
                element.dirty = true;
                this.mOnDidChangeTreeData.fire(element);
            }
        });
    }
    /**
     * Initialize the favorites and history information
     * @param log - Logger
     */
    initializeJobsTree(log) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = log;
            this.log.debug(localize(3, null));
            const lines = this.mHistory.readFavorites();
            lines.forEach((line) => {
                const profileName = line.substring(1, line.lastIndexOf("]"));
                const nodeName = (line.substring(line.indexOf(":") + 1, line.indexOf("{"))).trim();
                const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                try {
                    const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                    let favJob;
                    if (line.substring(line.indexOf("{") + 1, line.lastIndexOf("}")) === extension.JOBS_JOB_CONTEXT) {
                        favJob = new ZoweJobNode_1.Job(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.Collapsed, this.mFavoriteSession, core_1.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile), new JobDetail(nodeName), zosmfProfile);
                        favJob.contextValue = extension.JOBS_JOB_CONTEXT + extension.FAV_SUFFIX;
                        favJob.command = { command: "zowe.zosJobsSelectjob", title: "", arguments: [favJob] };
                    }
                    else { // for search
                        favJob = new ZoweJobNode_1.Job(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, core_1.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile), null, zosmfProfile);
                        favJob.command = { command: "zowe.jobs.search", title: "", arguments: [favJob] };
                        favJob.contextValue = extension.JOBS_SESSION_CONTEXT + extension.FAV_SUFFIX;
                    }
                    favJob.iconPath = utils_1.applyIcons(favJob);
                    this.mFavorites.push(favJob);
                }
                catch (e) {
                    const errMessage = localize(4, null) + profileName +
                        localize(5, null) +
                        localize(6, null) +
                        utils_1.getAppName(extension.ISTHEIA) + localize(7, null);
                    utils_1.errorHandling(e, null, errMessage);
                    return;
                }
            });
        });
    }
    /**
     * Adds a node to the Jobs favorites list
     *
     * @param {Job} node
     */
    addJobsFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const favJob = this.createJobsFavorite(node);
            if (!this.mFavorites.find((tempNode) => tempNode.label === favJob.label)) {
                this.mFavorites.push(favJob);
                utils_1.sortTreeItems(this.mFavorites, extension.JOBS_SESSION_CONTEXT + extension.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Adds a save search to the Jobs favorites list
     *
     * @param {Job} node
     */
    saveSearch(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const favSessionContext = extension.JOBS_SESSION_CONTEXT + extension.FAV_SUFFIX;
            const favJob = new ZoweJobNode_1.Job("[" + node.getSessionName() + "]: " +
                this.createSearchLabel(node.owner, node.prefix, node.searchId), vscode.TreeItemCollapsibleState.None, node.mParent, node.session, node.job, node.profile);
            favJob.owner = node.owner;
            favJob.prefix = node.prefix;
            favJob.searchId = node.searchId;
            favJob.contextValue = favSessionContext;
            favJob.command = { command: "zowe.jobs.search", title: "", arguments: [favJob] };
            favJob.iconPath = utils_1.applyIcons(favJob);
            if (!this.mFavorites.find((tempNode) => tempNode.label === favJob.label)) {
                this.mFavorites.push(favJob);
                utils_1.sortTreeItems(this.mFavorites, favSessionContext);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Removes a node from the favorites list
     *
     * @param {Job} node
     */
    removeJobsFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const startLength = this.mFavorites.length;
            this.mFavorites = this.mFavorites.filter((temp) => !((temp.label === node.label) && (temp.contextValue.startsWith(node.contextValue))));
            if (startLength !== this.mFavorites.length) {
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    updateFavorites() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = Object.assign({}, vscode.workspace.getConfiguration().get(ZosJobsProvider.persistenceSchema));
            if (settings.persistence) {
                settings.favorites = this.mFavorites.map((fav) => fav.label +
                    "{" + (fav.contextValue === extension.JOBS_JOB_CONTEXT + extension.FAV_SUFFIX ?
                    extension.JOBS_JOB_CONTEXT :
                    extension.JOBS_SESSION_CONTEXT) + "}");
                yield vscode.workspace.getConfiguration().update(ZosJobsProvider.persistenceSchema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
    /**
     * Prompts the user for search details to populate the [TreeView]{@link vscode.TreeView}
     *
     * @param {Job} node - The session node
     * @returns {Promise<void>}
     */
    searchPrompt(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let choice;
            let searchCriteria = "";
            const hasHistory = this.mHistory.getHistory().length > 0;
            let usrNme;
            let passWrd;
            let baseEncd;
            let sesNamePrompt;
            if (node.contextValue.endsWith(extension.FAV_SUFFIX)) {
                sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
            }
            else {
                sesNamePrompt = node.label;
            }
            if ((!node.session.ISession.user) || (!node.session.ISession.password)) {
                try {
                    const values = yield Profiles_1.Profiles.getInstance().promptCredentials(sesNamePrompt);
                    if (values !== undefined) {
                        usrNme = values[0];
                        passWrd = values[1];
                        baseEncd = values[2];
                    }
                }
                catch (error) {
                    yield utils_1.errorHandling(error, node.getProfileName(), error.message);
                }
                if (usrNme !== undefined && passWrd !== undefined && baseEncd !== undefined) {
                    node.session.ISession.user = usrNme;
                    node.session.ISession.password = passWrd;
                    node.session.ISession.base64EncodedAuth = baseEncd;
                    node.owner = usrNme;
                    this.validProfile = 0;
                }
                else {
                    return;
                }
                yield this.refreshElement(node);
                yield this.refresh();
            }
            else {
                this.validProfile = 0;
            }
            if (this.validProfile === 0) {
                if (node.contextValue === extension.JOBS_SESSION_CONTEXT) { // This is the profile object context
                    if (hasHistory) { // Check if user has created some history
                        const items = this.mHistory.getHistory().map((element) => new utils_1.FilterItem(element));
                        if (extension.ISTHEIA) { // Theia doesn't work properly when directly creating a QuickPick
                            const options1 = {
                                placeHolder: localize(8, null)
                            };
                            // get user selection
                            choice = (yield vscode.window.showQuickPick([this.createOwner, this.createId, ...items], options1));
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(9, null));
                                return;
                            }
                            searchCriteria = choice === this.createOwner || choice === this.createId ? "" : choice.label;
                        }
                        else { // VSCode route to create a QuickPick
                            const quickpick = vscode.window.createQuickPick();
                            quickpick.items = [this.createOwner, this.createId, ...items];
                            quickpick.placeholder = localize(10, null);
                            quickpick.ignoreFocusOut = true;
                            quickpick.show();
                            choice = yield utils_1.resolveQuickPickHelper(quickpick);
                            quickpick.hide();
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(11, null));
                                return;
                            }
                            if (choice instanceof utils_1.FilterDescriptor) {
                                if (quickpick.value.length > 0) {
                                    searchCriteria = this.interpretFreeform(quickpick.value);
                                }
                            }
                            else {
                                searchCriteria = choice.label;
                            }
                        }
                    }
                    if (!searchCriteria) { // Do we have anything to search with yet?
                        // if (searchCriteria === ZosJobsProvider.defaultDialogText) {
                        let options;
                        let owner;
                        let prefix;
                        let jobid;
                        // manually entering a search
                        if (!hasHistory || choice === this.createOwner) { // User has selected owner/prefix option
                            options = {
                                prompt: localize(12, null),
                                validateInput: (value) => (value.match(/ /g) ? localize(13, null) : ""),
                                value: node.owner
                            };
                            // get user input
                            owner = yield vscode.window.showInputBox(options);
                            if (owner === undefined) {
                                vscode.window.showInformationMessage(localize(14, null));
                                return;
                            }
                            if (!owner) {
                                owner = "*";
                            }
                            owner = owner.toUpperCase();
                            options = {
                                prompt: localize(15, null),
                                value: node.prefix
                            };
                            // get user input
                            prefix = yield vscode.window.showInputBox(options);
                            if (prefix === undefined) {
                                vscode.window.showInformationMessage(localize(16, null));
                                return;
                            }
                            if (!prefix) {
                                prefix = "*";
                            }
                            prefix = prefix.toUpperCase();
                            if (!hasHistory || choice === this.createId) {
                                options = {
                                    prompt: localize(17, null),
                                    value: node.searchId
                                };
                                // get user input
                                jobid = yield vscode.window.showInputBox(options);
                                if (jobid === undefined) {
                                    vscode.window.showInformationMessage(localize(18, null));
                                    return;
                                }
                            }
                        }
                        else { // User has selected JobId option
                            options = {
                                prompt: localize(19, null),
                                value: node.searchId
                            };
                            // get user input
                            jobid = yield vscode.window.showInputBox(options);
                            if (!jobid) {
                                vscode.window.showInformationMessage(localize(20, null));
                                return;
                            }
                        }
                        searchCriteria = this.createSearchLabel(owner, prefix, jobid);
                    }
                    this.applySearchLabelToNode(node, searchCriteria);
                }
                else {
                    // executing search from saved search in favorites
                    searchCriteria = node.label.trim().substring(node.label.trim().indexOf(":") + 2);
                    const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
                    const faveNode = node;
                    yield this.addSession(session);
                    node = this.mSessionNodes.find((tempNode) => tempNode.label.trim() === session);
                    if ((!node.session.ISession.user) || (!node.session.ISession.password)) {
                        node.session.ISession.user = faveNode.session.ISession.user;
                        node.session.ISession.password = faveNode.session.ISession.password;
                        node.session.ISession.base64EncodedAuth = faveNode.session.ISession.base64EncodedAuth;
                    }
                    this.applySearchLabelToNode(node, searchCriteria);
                }
                node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                node.iconPath = utils_1.applyIcons(node.getSessionNode(), extension.ICON_STATE_OPEN);
                utils_1.labelHack(node);
                node.dirty = true;
                this.refreshElement(node);
                this.addHistory(searchCriteria);
            }
        });
    }
    onDidChangeConfiguration(e) {
        return __awaiter(this, void 0, void 0, function* () {
            if (e.affectsConfiguration(ZosJobsProvider.persistenceSchema)) {
                const setting = Object.assign({}, vscode.workspace.getConfiguration().get(ZosJobsProvider.persistenceSchema));
                if (!setting.persistence) {
                    setting.favorites = [];
                    setting.history = [];
                    yield vscode.workspace.getConfiguration().update(ZosJobsProvider.persistenceSchema, setting, vscode.ConfigurationTarget.Global);
                }
            }
        });
    }
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            if (criteria) {
                this.mHistory.addHistory(criteria);
            }
        });
    }
    /**
     * Creates a display string to represent a search
     * @param owner - The owner search item
     * @param prefix - The job prefix search item
     * @param jobid - A specific jobid search item
     */
    createSearchLabel(owner, prefix, jobid) {
        let revisedCriteria = "";
        const alphaNumeric = new RegExp("^\w+$");
        if (jobid && !alphaNumeric.exec(jobid.trim())) {
            revisedCriteria = ZoweJobNode_1.Job.JobId + jobid.toUpperCase().trim();
        }
        else {
            if (owner) {
                revisedCriteria = ZoweJobNode_1.Job.Owner + owner.trim() + " ";
            }
            if (prefix) {
                revisedCriteria += ZoweJobNode_1.Job.Prefix + prefix.trim();
            }
        }
        return revisedCriteria.trim();
    }
    interpretFreeform(input) {
        let jobId;
        let owner;
        let prefix;
        // test if it's like our stored structure
        const criteria = input.split(" ");
        criteria.forEach((crit, index) => {
            if (crit.toUpperCase().indexOf(ZosJobsProvider.JobId.toUpperCase()) > -1 && criteria.length >= index + 2) {
                jobId = criteria[index + 1].trim().toUpperCase();
            }
            if (crit.toUpperCase().indexOf(ZosJobsProvider.Owner.toUpperCase()) > -1 && criteria.length >= index + 2) {
                owner = criteria[index + 1].trim().toUpperCase();
            }
            if (crit.toUpperCase().indexOf(ZosJobsProvider.Prefix.toUpperCase()) > -1 && criteria.length >= index + 2) {
                prefix = criteria[index + 1].trim().toUpperCase();
            }
        });
        // test for a jobId on it's own
        const jobPattern = new RegExp("[a-zA-Z]{3}[0-9]{5}");
        const jobs = jobPattern.exec(input);
        if (jobs && jobs.length > 0) {
            return this.createSearchLabel("*", "*", jobs[0]);
        }
        if (!owner && !prefix && !jobId) {
            const validPattern = new RegExp("[a-zA-Z0-9*]{2,8}");
            criteria.forEach((crit, index) => {
                if (index === 0 && validPattern.test(crit)) {
                    owner = crit.trim();
                }
                else if (index === 1 && validPattern.test(crit)) {
                    prefix = crit.trim();
                }
            });
        }
        return this.createSearchLabel(owner, prefix, jobId);
    }
    /**
     * Function that takes a search criteria and updates a search node based upon it
     * @param node - a Job node
     * @param storedSearch - The original search string
     */
    applySearchLabelToNode(node, storedSearch) {
        if (storedSearch) {
            node.searchId = "";
            node.owner = "*";
            node.prefix = "*";
            const criteria = storedSearch.split(" ");
            for (const crit of criteria) {
                let index = crit.indexOf(ZosJobsProvider.JobId);
                if (index > -1) {
                    index += ZosJobsProvider.JobId.length;
                    node.searchId = crit.substring(index).trim();
                }
                index = crit.indexOf(ZosJobsProvider.Owner);
                if (index > -1) {
                    index += ZosJobsProvider.Owner.length;
                    node.owner = crit.substring(index).trim();
                }
                index = crit.indexOf(ZosJobsProvider.Prefix);
                if (index > -1) {
                    index += ZosJobsProvider.Prefix.length;
                    node.prefix = crit.substring(index).trim();
                }
            }
        }
    }
    createJobsFavorite(node) {
        const favJob = new ZoweJobNode_1.Job("[" + node.getSessionName() + "]: " +
            node.label.substring(0, node.label.lastIndexOf(")") + 1), vscode.TreeItemCollapsibleState.Collapsed, node.mParent, node.session, node.job, node.profile);
        favJob.contextValue = extension.JOBS_JOB_CONTEXT + extension.FAV_SUFFIX;
        favJob.command = { command: "zowe.zosJobsSelectjob", title: "", arguments: [favJob] };
        favJob.iconPath = utils_1.applyIcons(favJob);
        return favJob;
    }
    /**
     * Adds a single session to the jobs tree
     *
     */
    addSingleSession(zosmfProfile) {
        return __awaiter(this, void 0, void 0, function* () {
            if (zosmfProfile) {
                // If session is already added, do nothing
                if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                    return;
                }
                // Uses loaded profile to create a zosmf session with brightside
                const session = core_1.ZosmfSession.createBasicZosmfSession(zosmfProfile.profile);
                // Creates ZoweNode to track new session and pushes it to mSessionNodes
                const node = new ZoweJobNode_1.Job(zosmfProfile.name, vscode.TreeItemCollapsibleState.Collapsed, null, session, null, zosmfProfile);
                node.contextValue = extension.JOBS_SESSION_CONTEXT;
                node.iconPath = utils_1.applyIcons(node);
                node.dirty = true;
                this.mSessionNodes.push(node);
                this.mHistory.addSession(zosmfProfile.name);
            }
        });
    }
}
exports.ZosJobsProvider = ZosJobsProvider;
ZosJobsProvider.JobId = "JobId:";
ZosJobsProvider.Owner = "Owner:";
ZosJobsProvider.Prefix = "Prefix:";
ZosJobsProvider.defaultDialogText = localize(21, null);
ZosJobsProvider.persistenceSchema = "Zowe-Jobs-Persistent";
/**
 * Helper class to allow generation of job details by history or favorite
 */
// tslint:disable-next-line: max-classes-per-file
class JobDetail {
    constructor(combined) {
        this.jobname = combined.substring(0, combined.indexOf("("));
        this.jobid = combined.substring(combined.indexOf("(") + 1, combined.indexOf(")"));
    }
}

//# sourceMappingURL=ZosJobsProvider.js.map
