"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
Object.defineProperty(exports, "__esModule", { value: true });
const ZoweExplorerZosmfApi_1 = require("./ZoweExplorerZosmfApi");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * The Zowe Explorer API Register singleton that gets exposed to other VS Code
 * extensions to contribute their implementations.
 * @export
 */
class ZoweExplorerApiRegister {
    /**
     * Private constructor that creates the singleton instance of ZoweExplorerApiRegister.
     * It automatically registers the zosmf implementation as it is the default for Zowe Explorer.
     */
    constructor() {
        // These are the different API registries currently available to extenders
        this.ussApiImplementations = new Map();
        this.mvsApiImplementations = new Map();
        this.jesApiImplementations = new Map();
        this.registerUssApi(new ZoweExplorerZosmfApi_1.ZosmfUssApi());
        this.registerMvsApi(new ZoweExplorerZosmfApi_1.ZosmfMvsApi());
        this.registerJesApi(new ZoweExplorerZosmfApi_1.ZosmfJesApi());
    }
    /**
     * Access the singleton instance.
     * @static
     * @returns {ZoweExplorerApiRegister} the ZoweExplorerApiRegister singleton instance
     */
    static getInstance() {
        return ZoweExplorerApiRegister.register;
    }
    /**
     * Static lookup of an API for USS for a given profile.
     * @private
     * @param {imperative.IProfileLoaded} a profile to be used with this instance of the API returned
     * @returns an instance of the API that uses the profile provided
     */
    static getUssApi(profile) {
        return ZoweExplorerApiRegister.getInstance().getUssApi(profile);
    }
    /**
     * Static lookup of an API for MVS for a given profile.
     * @private
     * @param {imperative.IProfileLoaded} a profile to be used with this instance of the API returned
     * @returns an instance of the API that uses the profile provided
     */
    static getMvsApi(profile) {
        return ZoweExplorerApiRegister.getInstance().getMvsApi(profile);
    }
    /**
     * Static lookup of an API for JES for a given profile.
     * @private
     * @param {imperative.IProfileLoaded} a profile to be used with this instance of the API returned
     * @returns an instance of the API that uses the profile provided
     */
    static getJesApi(profile) {
        return ZoweExplorerApiRegister.getInstance().getJesApi(profile);
    }
    // TODO: the redundant functions that follow could be done with generics, but as we are using
    // interfaces here that do not have type meta data it would require that they have type info
    // added. On the other these functions make the client code easier to read and understand.
    /**
     * Other VS Code extension need to call this to register their USS API implementation.
     * @param {ZoweExplorerApi.IUss} ussApi
     */
    registerUssApi(ussApi) {
        if (ussApi && ussApi.getProfileTypeName()) {
            this.ussApiImplementations.set(ussApi.getProfileTypeName(), ussApi);
        }
        else {
            throw new Error(localize(0, null));
        }
    }
    /**
     * Other VS Code extension need to call this to register their MVS API implementation.
     * @param {ZoweExplorerApi.IMvs} mvsApi
     */
    registerMvsApi(mvsApi) {
        if (mvsApi && mvsApi.getProfileTypeName()) {
            this.mvsApiImplementations.set(mvsApi.getProfileTypeName(), mvsApi);
        }
        else {
            throw new Error(localize(1, null));
        }
    }
    /**
     * Other VS Code extension need to call this to register their MVS API implementation.
     * @param {ZoweExplorerApi.IMvs} api
     */
    registerJesApi(jesApi) {
        if (jesApi && jesApi.getProfileTypeName()) {
            this.jesApiImplementations.set(jesApi.getProfileTypeName(), jesApi);
        }
        else {
            throw new Error(localize(2, null));
        }
    }
    /**
     * Get an array of all the registered APIs identified by the CLI profile type names,
     * such as ["zosmf", "zftp"].
     * @returns {string[]}
     */
    registeredApiTypes() {
        return [...new Set([...this.registeredUssApiTypes(), ...this.registeredMvsApiTypes(), ...this.registeredJesApiTypes()])];
    }
    /**
     * Get an array of all the registered USS APIs identified by the CLI profile types,
     * such as ["zosmf", "ftp"].
     * @returns {string[]}
     */
    registeredUssApiTypes() {
        return [...this.ussApiImplementations.keys()];
    }
    /**
     * Get an array of all the registered MVS APIs identified by the CLI profile types,
     * such as ["zosmf", "zftp"].
     * @returns {string[]}
     */
    registeredMvsApiTypes() {
        return [...this.mvsApiImplementations.keys()];
    }
    /**
     * Get an array of all the registered JES APIs identified by the CLI profile types,
     * such as ["zosmf", "zftp"].
     * @returns {string[]}
     */
    registeredJesApiTypes() {
        return [...this.jesApiImplementations.keys()];
    }
    /**
     * Lookup of an API implementation for USS for a given profile.
     * @param {imperative.IProfileLoaded} profile
     * @returns an instance of the API for the profile provided
     */
    getUssApi(profile) {
        if (profile && profile.type && this.registeredUssApiTypes().includes(profile.type)) {
            // create a clone of the API object that remembers the profile with which it was created
            const api = Object.create(this.ussApiImplementations.get(profile.type));
            api.profile = profile;
            return api;
        }
        else {
            throw new Error(localize(3, null) + profile.type);
        }
    }
    /**
     * Lookup of an API implementation for MVS for a given profile.
     * @param {imperative.IProfileLoaded} profile
     * @returns an instance of the API for the profile provided
     */
    getMvsApi(profile) {
        if (profile && profile.type && this.registeredMvsApiTypes().includes(profile.type)) {
            // create a clone of the API object that remembers the profile with which it was created
            const api = Object.create(this.mvsApiImplementations.get(profile.type));
            api.profile = profile;
            return api;
        }
        else {
            throw new Error(localize(4, null) + profile.type);
        }
    }
    /**
     * Lookup of an API implementation for JES for a given profile.
     * @param {imperative.IProfileLoaded} profile
     * @returns an instance of the API for the profile provided
     */
    getJesApi(profile) {
        if (profile && profile.type && this.registeredJesApiTypes().includes(profile.type)) {
            // create a clone of the API object that remembers the profile with which it was created
            const api = Object.create(this.jesApiImplementations.get(profile.type));
            api.profile = profile;
            return api;
        }
        else {
            throw new Error(localize(5, null) + profile.type);
        }
    }
}
exports.ZoweExplorerApiRegister = ZoweExplorerApiRegister;
/**
 * This object represents a collection of the APIs that get exposed to other VS Code
 * extensions that want to contribute alternative implementations such as alternative ways
 * of retrieving files and data from z/OS.
 */
ZoweExplorerApiRegister.register = new ZoweExplorerApiRegister();

//# sourceMappingURL=ZoweExplorerApiRegister.js.map
